package sun.awt.image.codec;

import java.awt.color.ColorSpace;
import java.awt.image.ColorModel;
import java.util.Enumeration;
import java.util.Vector;
import com.sun.image.codec.jpeg.JPEGDecodeParam;
import com.sun.image.codec.jpeg.JPEGHuffmanTable;
import com.sun.image.codec.jpeg.JPEGQTable;
import com.sun.image.codec.jpeg.JPEGEncodeParam;

public class JPEGParam implements JPEGEncodeParam, Cloneable
{
    private static int[] defComponents;
    private static int[][] stdCompMapping;
    private static int[][] stdSubsample;
    private int width;
    private int height;
    private int encodedColorID;
    private int numComponents;
    private byte[][][] appMarkers;
    private byte[][] comMarker;
    private boolean imageInfoValid;
    private boolean tableInfoValid;
    private int[] horizontalSubsampling;
    private int[] verticalSubsampling;
    private JPEGQTable[] qTables;
    private int[] qTableMapping;
    private JPEGHuffmanTable[] dcHuffTables;
    private int[] dcHuffMapping;
    private JPEGHuffmanTable[] acHuffTables;
    private int[] acHuffMapping;
    private int restartInterval;
    private static final int app0Length = 14;
    
    public JPEGParam(int n) {
        this(n, JPEGParam.defComponents[n]);
    }
    
    public JPEGParam(JPEGDecodeParam jpegDecodeParam) {
        this(jpegDecodeParam.getEncodedColorID(), jpegDecodeParam.getNumComponents());
        this.copy(jpegDecodeParam);
    }
    
    public JPEGParam(JPEGEncodeParam jpegEncodeParam) {
        this(jpegEncodeParam.getEncodedColorID(), jpegEncodeParam.getNumComponents());
        this.copy(jpegEncodeParam);
    }
    
    public JPEGParam(int defaults, int numComponents) {
        if (defaults != 0 && numComponents != JPEGParam.defComponents[defaults]) {
            throw new IllegalArgumentException("NumComponents not in sync with COLOR_ID");
        }
        this.qTables = new JPEGQTable[4];
        this.acHuffTables = new JPEGHuffmanTable[4];
        this.dcHuffTables = new JPEGHuffmanTable[4];
        for (int i = 0; i < 4; ++i) {
            this.qTables[i] = null;
            this.dcHuffTables[i] = null;
            this.acHuffTables[i] = null;
        }
        this.comMarker = (byte[][])null;
        this.appMarkers = new byte[16][][];
        this.numComponents = numComponents;
        this.setDefaults(defaults);
        return;
    }
    
    private void copy(JPEGDecodeParam jpegDecodeParam) {
        if (this.getEncodedColorID() != jpegDecodeParam.getEncodedColorID()) {
            throw new IllegalArgumentException("Argument to copy must match current COLOR_ID");
        }
        if (this.getNumComponents() != jpegDecodeParam.getNumComponents()) {
            throw new IllegalArgumentException("Argument to copy must match in number of components");
        }
        this.setWidth(jpegDecodeParam.getWidth());
        this.setHeight(jpegDecodeParam.getHeight());
        for (int i = 224; i < 239; ++i) {
            this.setMarkerData(i, copyArrays(jpegDecodeParam.getMarkerData(i)));
        }
        this.setMarkerData(254, copyArrays(jpegDecodeParam.getMarkerData(254)));
        this.setTableInfoValid(jpegDecodeParam.isTableInfoValid());
        this.setImageInfoValid(jpegDecodeParam.isImageInfoValid());
        this.setRestartInterval(jpegDecodeParam.getRestartInterval());
        for (int j = 0; j < 4; ++j) {
            this.setDCHuffmanTable(j, jpegDecodeParam.getDCHuffmanTable(j));
            this.setACHuffmanTable(j, jpegDecodeParam.getACHuffmanTable(j));
            this.setQTable(j, jpegDecodeParam.getQTable(j));
        }
        for (int k = 0; k < jpegDecodeParam.getNumComponents(); ++k) {
            this.setDCHuffmanComponentMapping(k, jpegDecodeParam.getDCHuffmanComponentMapping(k));
            this.setACHuffmanComponentMapping(k, jpegDecodeParam.getACHuffmanComponentMapping(k));
            this.setQTableComponentMapping(k, jpegDecodeParam.getQTableComponentMapping(k));
            this.setHorizontalSubsampling(k, jpegDecodeParam.getHorizontalSubsampling(k));
            this.setVerticalSubsampling(k, jpegDecodeParam.getVerticalSubsampling(k));
        }
        return;
    }
    
    private void copy(JPEGEncodeParam jpegEncodeParam) {
        this.copy(jpegEncodeParam);
    }
    
    protected void setDefaults(int encodedColorID) {
        this.encodedColorID = encodedColorID;
        this.restartInterval = 0;
        boolean n = false;
        switch (this.numComponents) {
            case 1: {
                if (this.encodedColorID == 1 || this.encodedColorID == 0) {
                    n = true;
                    break;
                }
                break;
            }
            case 3: {
                if (this.encodedColorID == 3) {
                    n = true;
                    break;
                }
                break;
            }
            case 4: {
                if (this.encodedColorID == 4) {
                    n = true;
                    break;
                }
                break;
            }
        }
        if (!(n == 0)) {
            this.addMarkerData(224, createDefaultAPP0Marker());
        }
        this.setTableInfoValid(true);
        this.setImageInfoValid(true);
        this.dcHuffTables[0] = JPEGHuffmanTable.StdDCLuminance;
        this.dcHuffTables[1] = JPEGHuffmanTable.StdDCChrominance;
        this.dcHuffMapping = new int[this.getNumComponents()];
        System.arraycopy(JPEGParam.stdCompMapping[this.encodedColorID], 0, this.dcHuffMapping, 0, this.getNumComponents());
        this.acHuffTables[0] = JPEGHuffmanTable.StdACLuminance;
        this.acHuffTables[1] = JPEGHuffmanTable.StdACChrominance;
        this.acHuffMapping = new int[this.getNumComponents()];
        System.arraycopy(JPEGParam.stdCompMapping[this.encodedColorID], 0, this.acHuffMapping, 0, this.getNumComponents());
        this.qTables[0] = JPEGQTable.StdLuminance.getScaledInstance(0.5f, true);
        this.qTables[1] = JPEGQTable.StdChrominance.getScaledInstance(0.5f, true);
        this.qTableMapping = new int[this.getNumComponents()];
        System.arraycopy(JPEGParam.stdCompMapping[this.encodedColorID], 0, this.qTableMapping, 0, this.getNumComponents());
        this.horizontalSubsampling = new int[this.getNumComponents()];
        System.arraycopy(JPEGParam.stdSubsample[this.encodedColorID], 0, this.horizontalSubsampling, 0, this.getNumComponents());
        this.verticalSubsampling = new int[this.getNumComponents()];
        System.arraycopy(JPEGParam.stdSubsample[this.encodedColorID], 0, this.verticalSubsampling, 0, this.getNumComponents());
    }
    
    public Object clone() {
        JPEGParam jpegParam = new JPEGParam(this.getEncodedColorID(), this.getNumComponents());
        jpegParam.copy(this);
        return jpegParam;
    }
    
    public int getWidth() {
        return this.width;
    }
    
    public int getHeight() {
        return this.height;
    }
    
    public void setWidth(int width) {
        this.width = width;
    }
    
    public void setHeight(int height) {
        this.height = height;
    }
    
    public int getHorizontalSubsampling(int n) {
        if (n < 0 || n >= this.getNumComponents()) {
            throw new IllegalArgumentException("Component must be between 0 and number of components");
        }
        return this.horizontalSubsampling[n];
    }
    
    public int getVerticalSubsampling(int n) {
        if (n < 0 || n >= this.getNumComponents()) {
            throw new IllegalArgumentException("Component must be between 0 and number of components");
        }
        return this.verticalSubsampling[n];
    }
    
    public void setHorizontalSubsampling(int i, int j) {
        if (i < 0 || i >= this.getNumComponents()) {
            throw new IllegalArgumentException("Component must be between 0 and number of components: " + i);
        }
        if (j <= 0) {
            throw new IllegalArgumentException("SubSample factor must be positive: " + j);
        }
        this.horizontalSubsampling[i] = j;
        return;
    }
    
    public void setVerticalSubsampling(int n, int n2) {
        if (n < 0 || n >= this.getNumComponents()) {
            throw new IllegalArgumentException("Component must be between 0 and number of components");
        }
        if (n2 <= 0) {
            throw new IllegalArgumentException("SubSample factor must be positive.");
        }
        this.verticalSubsampling[n] = n2;
        return;
    }
    
    public JPEGQTable getQTable(int n) {
        if (n < 0 || n >= 4) {
            throw new IllegalArgumentException("tableNum must be between 0 and 3.");
        }
        return this.qTables[n];
    }
    
    public JPEGQTable getQTableForComponent(int n) {
        if (n < 0 || n >= this.qTableMapping.length) {
            throw new IllegalArgumentException("Component must be between 0 and number of components");
        }
        return this.getQTable(this.qTableMapping[n]);
    }
    
    public JPEGHuffmanTable getDCHuffmanTable(int n) {
        if (n < 0 || n >= 4) {
            throw new IllegalArgumentException("tableNum must be 0-3.");
        }
        return this.dcHuffTables[n];
    }
    
    public JPEGHuffmanTable getDCHuffmanTableForComponent(int n) {
        if (n < 0 || n >= this.dcHuffMapping.length) {
            throw new IllegalArgumentException("Component must be between 0 and number of components");
        }
        return this.getDCHuffmanTable(this.dcHuffMapping[n]);
    }
    
    public JPEGHuffmanTable getACHuffmanTable(int n) {
        if (n < 0 || n >= 4) {
            throw new IllegalArgumentException("tableNum must be 0-3.");
        }
        return this.acHuffTables[n];
    }
    
    public JPEGHuffmanTable getACHuffmanTableForComponent(int n) {
        if (n < 0 || n >= this.acHuffMapping.length) {
            throw new IllegalArgumentException("Component must be between 0 and number of components");
        }
        return this.getACHuffmanTable(this.acHuffMapping[n]);
    }
    
    public void setQTable(int n, JPEGQTable jpegqTable) {
        if (n < 0 || n >= 4) {
            throw new IllegalArgumentException("tableNum must be between 0 and 3.");
        }
        this.qTables[n] = jpegqTable;
        return;
    }
    
    public void setDCHuffmanTable(int n, JPEGHuffmanTable jpegHuffmanTable) {
        if (n < 0 || n >= 4) {
            throw new IllegalArgumentException("tableNum must be 0, 1, 2, or 3.");
        }
        this.dcHuffTables[n] = jpegHuffmanTable;
        return;
    }
    
    public void setACHuffmanTable(int n, JPEGHuffmanTable jpegHuffmanTable) {
        if (n < 0 || n >= 4) {
            throw new IllegalArgumentException("tableNum must be 0, 1, 2, or 3.");
        }
        this.acHuffTables[n] = jpegHuffmanTable;
        return;
    }
    
    public int getDCHuffmanComponentMapping(int n) {
        if (n < 0 || n >= this.getNumComponents()) {
            throw new IllegalArgumentException("Requested Component doesn't exist.");
        }
        return this.dcHuffMapping[n];
    }
    
    public int getACHuffmanComponentMapping(int n) {
        if (n < 0 || n >= this.getNumComponents()) {
            throw new IllegalArgumentException("Requested Component doesn't exist.");
        }
        return this.acHuffMapping[n];
    }
    
    public int getQTableComponentMapping(int n) {
        if (n < 0 || n >= this.getNumComponents()) {
            throw new IllegalArgumentException("Requested Component doesn't exist.");
        }
        return this.qTableMapping[n];
    }
    
    public void setDCHuffmanComponentMapping(int n, int n2) {
        if (n < 0 || n >= this.getNumComponents()) {
            throw new IllegalArgumentException("Given Component doesn't exist.");
        }
        if (n2 < 0 || n2 >= 4) {
            throw new IllegalArgumentException("Tables must be 0, 1, 2, or 3.");
        }
        this.dcHuffMapping[n] = n2;
        return;
    }
    
    public void setACHuffmanComponentMapping(int n, int n2) {
        if (n < 0 || n >= this.getNumComponents()) {
            throw new IllegalArgumentException("Given Component doesn't exist.");
        }
        if (n2 < 0 || n2 >= 4) {
            throw new IllegalArgumentException("Tables must be 0, 1, 2, or 3.");
        }
        this.acHuffMapping[n] = n2;
        return;
    }
    
    public void setQTableComponentMapping(int n, int n2) {
        if (n < 0 || n >= this.getNumComponents()) {
            throw new IllegalArgumentException("Given Component doesn't exist.");
        }
        if (n2 < 0 || n2 >= 4) {
            throw new IllegalArgumentException("Tables must be 0, 1, 2, or 3.");
        }
        this.qTableMapping[n] = n2;
        return;
    }
    
    public boolean isImageInfoValid() {
        return this.imageInfoValid;
    }
    
    public void setImageInfoValid(boolean imageInfoValid) {
        this.imageInfoValid = imageInfoValid;
    }
    
    public boolean isTableInfoValid() {
        return this.tableInfoValid;
    }
    
    public void setTableInfoValid(boolean tableInfoValid) {
        this.tableInfoValid = tableInfoValid;
    }
    
    public boolean getMarker(int i) {
        byte[][] array = (byte[][])null;
        byte[][] comMarker;
        if (i == 254) {
            comMarker = this.comMarker;
        }
        else if (i >= 224 && i <= 239) {
            comMarker = this.appMarkers[i - 224];
        }
        else {
            throw new IllegalArgumentException("Invalid Marker ID:" + i);
        }
        return comMarker != null && comMarker.length != 0;
    }
    
    public byte[][] getMarkerData(int i) {
        if (i == 254) {
            return this.comMarker;
        }
        if (i >= 224 && i <= 239) {
            return this.appMarkers[i - 224];
        }
        throw new IllegalArgumentException("Invalid Marker ID:" + i);
    }
    
    public void setMarkerData(int i, byte[][] comMarker) {
        if (i == 254) {
            this.comMarker = comMarker;
        }
        else if (i >= 224 && i <= 239) {
            this.appMarkers[i - 224] = comMarker;
        }
        else {
            throw new IllegalArgumentException("Invalid Marker ID:" + i);
        }
    }
    
    public void addMarkerData(int i, byte[] array) {
        if (array == null) {
            return;
        }
        if (i == 254) {
            this.comMarker = appendArray(this.comMarker, array);
        }
        else if (i >= 224 && i <= 239) {
            this.appMarkers[i - 224] = appendArray(this.appMarkers[i - 224], array);
        }
        else {
            throw new IllegalArgumentException("Invalid Marker ID:" + i);
        }
        return;
    }
    
    public int getEncodedColorID() {
        return this.encodedColorID;
    }
    
    public int getNumComponents() {
        return this.numComponents;
    }
    
    public static int getNumComponents(int n) {
        if (n < 0 || n >= 12) {
            throw new IllegalArgumentException("Invalid JPEGColorID.");
        }
        return JPEGParam.defComponents[n];
    }
    
    public int getRestartInterval() {
        return this.restartInterval;
    }
    
    public void setRestartInterval(int restartInterval) {
        this.restartInterval = restartInterval;
    }
    
    public int getDensityUnit() {
        if (!this.getMarker(224)) {
            throw new IllegalArgumentException("No APP0 marker present");
        }
        byte[] app0 = this.findAPP0();
        if (app0 == null) {
            throw new IllegalArgumentException("Can't understand APP0 marker that is present");
        }
        return app0[7];
    }
    
    public int getXDensity() {
        if (!this.getMarker(224)) {
            throw new IllegalArgumentException("No APP0 marker present");
        }
        byte[] app0 = this.findAPP0();
        if (app0 == null) {
            throw new IllegalArgumentException("Can't understand APP0 marker that is present");
        }
        return app0[8] << 8 | (app0[9] & 0xFF);
    }
    
    public int getYDensity() {
        if (!this.getMarker(224)) {
            throw new IllegalArgumentException("No APP0 marker present");
        }
        byte[] app0 = this.findAPP0();
        if (app0 == null) {
            throw new IllegalArgumentException("Can't understand APP0 marker that is present");
        }
        return app0[10] << 8 | (app0[11] & 0xFF);
    }
    
    public void setDensityUnit(int n) {
        byte[] array;
        if (!this.getMarker(224)) {
            array = createDefaultAPP0Marker();
            this.addMarkerData(224, array);
        }
        else {
            array = this.findAPP0();
            if (array == null) {
                throw new IllegalArgumentException("Can't understand APP0 marker that is present");
            }
        }
        array[7] = (byte)n;
    }
    
    public void setXDensity(int n) {
        byte[] array;
        if (!this.getMarker(224)) {
            array = createDefaultAPP0Marker();
            this.addMarkerData(224, array);
        }
        else {
            array = this.findAPP0();
            if (array == null) {
                throw new IllegalArgumentException("Can't understand APP0 marker that is present");
            }
        }
        array[8] = (byte)(n >>> 8 & 0xFF);
        array[9] = (byte)(n & 0xFF);
    }
    
    public void setYDensity(int n) {
        byte[] array;
        if (!this.getMarker(224)) {
            array = createDefaultAPP0Marker();
            this.addMarkerData(224, array);
        }
        else {
            array = this.findAPP0();
            if (array == null) {
                throw new IllegalArgumentException("Can't understand APP0 marker that is present");
            }
        }
        array[10] = (byte)(n >>> 8 & 0xFF);
        array[11] = (byte)(n & 0xFF);
    }
    
    public void setQuality(float n, boolean b) {
        double n2 = (double)n;
        if (n2 <= 0.01) {
            n2 = 0.01;
        }
        if (n2 > 1.0) {
            n2 = 1.0;
        }
        double n3;
        if (n2 < 0.5) {
            n3 = 0.5 / n2;
        }
        else {
            n3 = 2.0 - n2 * 2.0;
        }
        this.qTableMapping = new int[this.getNumComponents()];
        System.arraycopy(JPEGParam.stdCompMapping[this.encodedColorID], 0, this.qTableMapping, 0, this.getNumComponents());
        this.qTables[0] = JPEGQTable.StdLuminance.getScaledInstance((float)n3, b);
        this.qTables[1] = JPEGQTable.StdChrominance.getScaledInstance((float)n3, b);
        this.qTables[2] = null;
        this.qTables[3] = null;
    }
    
    byte[] findAPP0() {
        byte[][] array = (byte[][])null;
        byte[][] markerData = this.getMarkerData(224);
        if (markerData == null) {
            return null;
        }
        int i = 0;
        while (i < markerData.length) {
            if (markerData[i] != null && checkAPP0(markerData[i])) {
                return markerData[i];
            }
            ++i;
        }
        return null;
    }
    
    static boolean checkAPP0(byte[] array) {
        return array.length >= 14 && array[0] == 74 && array[1] == 70 && array[2] == 73 && array[3] == 70 && array[4] == 0 && array[5] >= 1;
    }
    
    static byte[] createDefaultAPP0Marker() {
        return new byte[] { 74, 70, 73, 70, 0, 1, 1, 0, 0, 1, 0, 1, 0, 0 };
    }
    
    static byte[] copyArray(byte[] array) {
        if (array == null) {
            return null;
        }
        byte[] array2 = new byte[array.length];
        System.arraycopy(array, 0, array2, 0, array.length);
        return array2;
    }
    
    static byte[][] copyArrays(byte[][] array) {
        if (array == null) {
            return (byte[][])null;
        }
        byte[][] array2 = new byte[array.length][];
        for (int i = 0; i < array.length; ++i) {
            if (array[i] != null) {
                array2[i] = copyArray(array[i]);
            }
        }
        return array2;
    }
    
    static byte[][] appendArray(byte[][] array, byte[] array2) {
        int length = 0;
        if (array != null) {
            length = array.length;
        }
        byte[][] array3 = new byte[length + 1][];
        for (int i = 0; i < length; ++i) {
            array3[i] = array[i];
        }
        if (array2 != null) {
            array3[length] = copyArray(array2);
            return array3;
        }
        return array3;
    }
    
    static byte[][] buildArray(Vector vector) {
        if (vector == null) {
            return (byte[][])null;
        }
        int n = 0;
        byte[][] array = new byte[vector.size()][];
        Enumeration<Object> elements = vector.elements();
        while (elements.hasMoreElements()) {
            byte[] array2 = (byte[])(byte[])elements.nextElement();
            if (array2 != null) {
                array[n++] = copyArray(array2);
            }
            else {
                continue;
            }
        }
        return array;
    }
    
    public static int getDefaultColorId(ColorModel colorModel) {
        int hasAlpha = colorModel.hasAlpha();
        ColorSpace colorSpace = colorModel.getColorSpace();
        ColorSpace instance = null;
        switch (colorSpace.getType()) {
            case 6: {
                return 1;
            }
            case 5: {
                if (!(hasAlpha == 0)) {
                    return 7;
                }
                return 3;
            }
            case 3: {
                if (instance == null) {
                    try {
                        instance = ColorSpace.getInstance(1002);
                    }
                    catch (IllegalArgumentException ex) {}
                }
                if (colorSpace == instance) {
                    return hasAlpha ? 10 : 5;
                }
                return hasAlpha ? 7 : 3;
            }
            case 9: {
                return 4;
            }
            default: {
                return 0;
            }
        }
    }
    
    static {
        JPEGParam.defComponents = new int[] { -1, 1, 3, 3, 4, 3, 4, 4, 4, 4, 4, 4 };
        JPEGParam.stdCompMapping = new int[][] { { 0, 0, 0, 0 }, { 0 }, { 0, 0, 0 }, { 0, 1, 1 }, { 0, 0, 0, 0 }, { 0, 1, 1 }, { 0, 0, 0, 0 }, { 0, 1, 1, 0 }, { 0, 0, 0, 0 }, { 0, 1, 1, 0 }, { 0, 1, 1, 0 }, { 0, 1, 1, 0 } };
        JPEGParam.stdSubsample = new int[][] { { 1, 1, 1, 1 }, { 1 }, { 1, 1, 1 }, { 1, 2, 2 }, { 1, 1, 1, 1 }, { 1, 2, 2 }, { 1, 1, 1, 1 }, { 1, 2, 2, 1 }, { 1, 1, 1, 1 }, { 1, 2, 2, 1 }, { 1, 2, 2, 1 }, { 1, 2, 2, 1 } };
    }
}
